import os, sys, math, random, time, pygame
from pygame.locals import *
from pygame.compat import geterror

import const, util, image, var, item, explosion

class Missile(pygame.sprite.DirtySprite):
    def __init__(self, x, y):
        pygame.sprite.DirtySprite.__init__(self)
        self.images = image.images.missile
        self.image = self.images.subsurface(16, 0, 16, 20);
        self.rect = self.image.get_rect()
        self.rect[0] = x - self.rect[2]/2
        self.rect[1] = y - self.rect[3]
        self.vel = -3      #only vertical velocity

    def update(self):
        self.dirty = 1
        self.rect[1] += self.vel
        for brick in list(var.game.bricks):
            if brick.collides(self):
                if brick.max_hits != 0 and brick.hit():
                    var.game.add_bonus(brick.rect[0] + brick.rect[2]/2, brick.rect[1] + brick.rect[3])
                    brick.die()    
                    var.game.update_score(1)
                var.game.allsprites.remove(self)
                return
        if self.rect[1] <= const.SCREEN_Y0:  #hit the top of the screen
            var.game.allsprites.remove(self)

class SpiderMissile(pygame.sprite.DirtySprite):
    def __init__(self, r):
        pygame.sprite.DirtySprite.__init__(self)
        self.image = image.images.spider_missile
        self.rect = self.image.get_rect()
        self.rect[0] = r[0] + r[2] / 2
        self.rect[1] = r[1] + (r[3] + self.rect[3]) / 2
        self.vel = 3

    def update(self):
        self.dirty = 1
        self.rect[1] += self.vel
        if pygame.sprite.collide_rect(self, var.game.paddle):
            var.game.paddle.die_maybe()
            e = explosion.Explosion(self.rect, True, False)
            return
        if self.rect[1] >= const.SCREEN_HEIGHT:
            var.game.allsprites.remove(self)            
        
class Bomb(pygame.sprite.DirtySprite):
    def __init__(self, countdown):
        pygame.sprite.DirtySprite.__init__(self)
        self.images = image.images.bomb
        self.image = self.images[0]
        self.rect = self.image.get_rect()
        self.radius = self.rect[2]/ 2
        self.vel = [0, -3] 
        self.frame = 0
        self.delay = 0
        self.max_delay = 8
        self.countdown = countdown
        self.on_paddle = True
        self.pos = [0, 0]
        
    def update(self):
        self.countdown -= 1
        if self.countdown <= 0:
            self.explode()
            return
        self.dirty = 1
        self.delay += 1
        if self.delay > self.max_delay:
            self.frame += 1
            if self.frame >= 4:
                self.frame = 0
            self.image = self.images[self.frame]
        if self.on_paddle:    
            self.pos = [var.game.paddle.pos, const.PADDLE_Y - var.game.paddle.height - self.radius ]
            self.rect[0], self.rect[1] = self.pos[0] - self.radius, self.pos[1] - self.radius
        else:
            #bounces from the walls like the ball
            posx = self.pos[0] + self.vel[0]
            posy = self.pos[1] + self.vel[1]
            if posx <= self.radius or posx >= const.SCREEN_WIDTH - self.radius:
                self.vel[0] = -self.vel[0]
                posx = self.pos[0] + self.vel[0]
            if posy <= const.SCREEN_Y0 + self.radius:
                self.vel[1] = -self.vel[1]
                posy = self.pos[1] + self.vel[1]
            elif posy >= const.SCREEN_HEIGHT - self.radius:
                posy = const.SCREEN_HEIGHT - self.radius
                self.vel[1] = -self.vel[1]
            self.pos[0], self.pos[1] = posx, posy 
            self.rect[0] = self.pos[0] - self.radius
            self.rect[1] = self.pos[1] - self.radius

    def release(self):
        self.on_paddle = False
        
    def explode(self):
        e = explosion.Explosion(self.rect, False, False)
        for brick in list(var.game.bricks):
            if pygame.sprite.collide_circle_ratio(3.5)(self, brick):
                brick.die()
                var.game.update_score(1)
        var.game.allsprites.remove(self)
        var.game.bomb = None
